function(set_git_version)
	find_package(Git)

	if (NOT Git_FOUND)
		message ( "[Git] Git not found" )
	endif()

	execute_process (
		COMMAND ${GIT_EXECUTABLE} describe --tags --match "v*" HEAD
		WORKING_DIRECTORY ${CMAKE_SOURCE_DIR}
		OUTPUT_VARIABLE ALIB_GIT_DESCRIBE
		RESULT_VARIABLE RET_CODE
		OUTPUT_STRIP_TRAILING_WHITESPACE
		)

	if("${RET_CODE}" STREQUAL "0")
		message ( "[Git] Describe: '${ALIB_GIT_DESCRIBE}'" )
		SET ( ALIB_GIT_DESCRIBE "${ALIB_GIT_DESCRIBE}" PARENT_SCOPE )
	else()
		message ( WARNING "[Git] Failed retrieving current commit (return code ${RET_CODE})" )
		SET ( ALIB_GIT_DESCRIBE "<unknown commit>" PARENT_SCOPE )
		return()
	endif()

	execute_process (
		COMMAND ${GIT_EXECUTABLE} rev-parse --abbrev-ref HEAD
		WORKING_DIRECTORY ${CMAKE_SOURCE_DIR}
		OUTPUT_VARIABLE ALIB_GIT_BRANCH
		RESULT_VARIABLE RET_CODE
		OUTPUT_STRIP_TRAILING_WHITESPACE
		)

	if("${RET_CODE}" STREQUAL "0")
		message ( "[Git] Branch: '${ALIB_GIT_BRANCH}'" )
		SET ( ALIB_GIT_BRANCH "${ALIB_GIT_BRANCH}" PARENT_SCOPE )
	else()
		message ( WARNING "[Git] Failed retrieving current branch (return code ${RET_CODE})" )
		SET ( ALIB_GIT_BRANCH "<unknown branch>" PARENT_SCOPE )
		return()
	endif()
endfunction(set_git_version)

function(set_git_snapshot_version)
	set(ALIB_SNAPSHOT_VERSION ${ALIB_GIT_DESCRIBE})
	string(REGEX REPLACE "^v" "" ALIB_SNAPSHOT_VERSION ${ALIB_SNAPSHOT_VERSION})
	string(REGEX REPLACE "\([^-]*-g\)" "r\\1" ALIB_SNAPSHOT_VERSION ${ALIB_SNAPSHOT_VERSION})
	string(REPLACE "-" "." ALIB_SNAPSHOT_VERSION ${ALIB_SNAPSHOT_VERSION})
	set(ALIB_SNAPSHOT_VERSION "${ALIB_SNAPSHOT_VERSION}" PARENT_SCOPE )
	message("[Git] Snapshot version: ${ALIB_SNAPSHOT_VERSION}")
endfunction(set_git_snapshot_version)
