/*
 * Prompt.h
 *
 *  Created on: 20. 3. 2017
 *	  Author: Jan Travnicek
 */

#ifndef _AQL_PROMPT_H_
#define _AQL_PROMPT_H_

#include <string>

#include <alib/exception>

#include <lexer/Lexer.h>
#include <parser/Parser.h>

#include <global/GlobalData.h>

#include "HistoryRegister.h"
#include "ReadlineCharSequence.h"

class Prompt {
	std::string m_history_file;

	cli::Environment m_environment;

	Prompt ( cli::Environment environment );

public:
	Prompt ( const Prompt & ) = delete;

	Prompt ( Prompt && ) = delete;

	Prompt & operator = ( const Prompt & ) = delete;

	Prompt & operator = ( Prompt && ) = delete;

	~Prompt ( );

	static Prompt & getPrompt ( ) {
		static Prompt instance { cli::Environment ( ) };
		return instance;
	}

	cli::Command::Result run ( );

	template < class T >
	cli::Command::Result execute_line ( T && charSequence, bool allowHistory ) {
		try {
			cli::Parser parser = cli::Parser ( cli::Lexer ( std::forward < T && > ( charSequence ) ) );
			HistoryRegister historyRegister ( parser.getLexer ( ), allowHistory );
			return parser.parse ( )->run ( m_environment );
		} catch ( ... ) {
			alib::ExceptionHandler::handle ( common::Streams::err );
			return cli::Command::Result::EXCEPTION;
		}
	}

	cli::Environment & getEnvironment ( ) {
		return m_environment;
	}

	const std::string & getHistoryFile ( ) const {
		return m_history_file;
	}
};

#endif /* _AQL_PROMPT_H_ */
