# $GIT_ROOT
# $TYPE (snapshot/stable)
# $CWD

before () {
	# prepare base-devel. gpg, git
	pacman -Suy --noconfirm --needed git gnupg sudo base-devel namcap
	useradd builder -m && passwd -d builder && printf 'builder ALL=(ALL) ALL\n' | tee -a /etc/sudoers

	export -f gpg_import
	su builder -c '/bin/sh -c gpg_import'

	echo "MAKEFLAGS=\"-j$(grep -c processor /proc/cpuinfo)\"" >> /etc/makepkg.conf
}

package () {
	set -e

	# prepare PKGBUILD
	cp $GIT_ROOT/extra/scripts/CD/distrofiles/arch/$TYPE/PKGBUILD .
	chown builder:builder PKGBUILD

	# makepkg Outputs:
	# - FOO.pkg.tar.xz
	# - FOO.pkg.tar.xz.sig
	sudo -u builder -- makepkg --syncdeps --noconfirm --sign --key $GPG_KEYID

	set +e
}

deploy () {
	set -e
	MOUNTPOINTS="-v /srv/repository/archlinux/x86_64:/repository -v$(pwd):/build -v/root/.gnupg/:/root/.gnupg/"
	DOCKER_IMAGE="archlinux/base"
	DOCKER_CMD="docker run --rm $MOUNTPOINTS $DOCKER_IMAGE"

	REPO_DIR="/repository"
	REPO_DB="/repository/algorithms-library.db.tar.gz"

	# docker login gitlab.fit.cvut.cz:5000 -u "$DOCKER_REGISTRY_USERNAME" -p "$DOCKER_REGISTRY_TOKEN"
    docker login -u gitlab-ci-token -p $CI_JOB_TOKEN $CI_REGISTRY
	docker pull "$DOCKER_IMAGE:latest"

	if [[ $TYPE == "snapshot" ]]; then
		REMOVE="algorithms-library-gui-snapshot algorithms-library-snapshot"
	else
		REMOVE="algorithms-library-gui algorithms-library"
	fi

	$DOCKER_CMD pacman -Suy --noconfirm --needed gnupg grep
	for file in *.tar.xz; do
		$DOCKER_CMD bash -c "cd $REPO_DIR && [[ -e $REPO_DB ]] && repo-remove -v -s -k $GPG_KEYID $REPO_DB $REMOVE || true"
		$DOCKER_CMD bash -c "for file in /build/*.tar.xz; do cp \$file $REPO_DIR/ && cp \$file.sig $REPO_DIR/ && repo-add -R -v -s -k $GPG_KEYID $REPO_DB $(basename "\$file"); done"
	done

	set +e
}
