before () {
	apt-get update
	apt-get install -y dh-make debsigs devscripts build-essential git gnupg
	gpg_import

	apt-get install -y cmake libxml2-dev libtclap-dev libreadline-dev qtbase5-dev graphviz-dev libjsoncpp-dev
}

package () {
	set -e

	# create orig file
	cp $ARCHIVE ${ARCHIVE%.tar.gz}.orig.tar.gz

	# extract sources and build
	tar xzvf $ARCHIVE
	pushd $ARCHIVE_DIR

	cp -r $GIT_ROOT/extra/scripts/CD/distrofiles/deb/$TYPE/debian .

	if [[ -z "$DISTRIBUTION" ]]; then
		echo "Distribution variable not set"
		exit 1
	fi

	# snapshot changelog update
	if [[ $TYPE == "snapshot" ]]; then
		CHANGELOG=$(cat debian/changelog)
		cat > debian/changelog <<DOC
algorithms-library ($VERSION~$DISTRIBUTION) unstable; urgency=medium

  * Snapshot release changelog bump.

 -- Tomáš Pecka <peckato1@fit.cvut.cz>  $(date "+%a, %d %b %Y %H:%M:%S %z")

$CHANGELOG
DOC
	fi

	debuild -b -us -k$GPG_KEYID -j$(grep -c processor /proc/cpuinfo)
	popd

	set +e
}

deploy () {
	set -e
	# comes from gitlab-ci:
	# DISTRIBUTION=

	MOUNTPOINTS="-v /srv/repository/debian:/repository -v$(pwd):/build -v/root/.gnupg/:/root/.gnupg/"
	DOCKER_IMAGE="gitlab.fit.cvut.cz:5000/algorithms-library-toolkit/ci-docker-images/repository/aptly"
	DOCKER_CMD="docker run --rm $MOUNTPOINTS $DOCKER_IMAGE"

	SNAPSHOT="snapshot-$DISTRIBUTION"

	if [[ $TYPE == "snapshot" ]]; then
		PACKAGE_REMOVE="algorithms-library-snapshot algorithms-library-dev-snapshot algorithms-library-gui-snapshot"
		PACKAGE_DBGSYM="algorithms-library-snapshot-dbgsym algorithms-library-gui-snapshot-dbgsym"
	else
		PACKAGE_REMOVE="algorithms-library algorithms-library-dev algorithms-library-gui"
		PACKAGE_DBGSYM="algorithms-library-dbgsym algorithms-library-gui-dbgsym"
	fi

	# docker login gitlab.fit.cvut.cz:5000 -u "$DOCKER_REGISTRY_USERNAME" -p "$DOCKER_REGISTRY_TOKEN"
    docker login -u gitlab-ci-token -p $CI_JOB_TOKEN $CI_REGISTRY
	docker pull "$DOCKER_IMAGE:latest"

	$DOCKER_CMD aptly repo create $DISTRIBUTION || true
	$DOCKER_CMD aptly publish drop "$DISTRIBUTION" filesystem:dir: || true
	$DOCKER_CMD aptly snapshot drop "$SNAPSHOT" || true
	$DOCKER_CMD aptly repo remove $DISTRIBUTION $PACKAGE_REMOVE || true
	$DOCKER_CMD bash -c "aptly repo include -repo="$DISTRIBUTION" /build/*.changes"
	$DOCKER_CMD aptly repo remove $DISTRIBUTION $PACKAGE_DBGSYM
	$DOCKER_CMD aptly repo show -with-packages "$DISTRIBUTION"
	$DOCKER_CMD aptly snapshot create -architectures=amd64 "$SNAPSHOT" from repo "$DISTRIBUTION"
	$DOCKER_CMD aptly publish snapshot -architectures=amd64 -batch -gpg-key="$GPG_KEYID" -distribution=$DISTRIBUTION $SNAPSHOT filesystem:dir:
	set +e
}
