before () {
	:;
}

package () {
	set -e

	IMAGE_CLI=$CI_REGISTRY_IMAGE/cli
	IMAGE_GUI=$CI_REGISTRY_IMAGE/gui
	IMAGE_ALL=$CI_REGISTRY_IMAGE
	TAG_SNAPSHOT=snapshot
	TAG_STABLE=$CI_COMMIT_REF_NAME
	TAG_LATEST=latest

	BUILD_ALL=all_build
	BUILD_CLI=cli_build

	if [[ $TYPE == "snapshot" ]]; then
		DOCKERFILE="Dockerfile.snapshot"
	elif [[ $TYPE == "stable" ]]; then
		DOCKERFILE="Dockerfile.stable"
	fi

	docker build --target=deploy-cli -f $GIT_ROOT/extra/docker/$DOCKERFILE -t $BUILD_CLI .
	docker build --target=deploy-all -f $GIT_ROOT/extra/docker/$DOCKERFILE -t $BUILD_ALL .

	docker run $BUILD_CLI /usr/bin/aql2 --help
	docker run $BUILD_ALL /usr/bin/aql2 --help

	if [[ $TYPE == "snapshot" ]]; then
		docker tag "$BUILD_CLI" "$IMAGE_CLI:$TAG_SNAPSHOT"
		docker tag "$BUILD_ALL" "$IMAGE_GUI:$TAG_SNAPSHOT" # gui-version is now only a symlink to all-version
		docker tag "$BUILD_ALL" "$IMAGE_ALL:$TAG_SNAPSHOT"

		docker save -o images.tar \
			"$IMAGE_CLI:$TAG_SNAPSHOT" \
			"$IMAGE_GUI:$TAG_SNAPSHOT" \
			"$IMAGE_ALL:$TAG_SNAPSHOT"

	elif [[ $TYPE == "stable" ]]; then
		docker tag "$BUILD_CLI" "$IMAGE_CLI:$TAG_STABLE"
		docker tag "$BUILD_CLI" "$IMAGE_CLI:$TAG_LATEST"
		docker tag "$BUILD_ALL" "$IMAGE_GUI:$TAG_STABLE" # gui-version is now only a symlink to all-version
		docker tag "$BUILD_ALL" "$IMAGE_GUI:$TAG_LATEST" # gui-version is now only a symlink to all-version
		docker tag "$BUILD_ALL" "$IMAGE_ALL:$TAG_STABLE"
		docker tag "$BUILD_ALL" "$IMAGE_ALL:$TAG_LATEST"

		docker save -o images.tar \
			"$IMAGE_CLI:$TAG_STABLE" "$IMAGE_CLI:$TAG_LATEST" \
			"$IMAGE_GUI:$TAG_STABLE" "$IMAGE_GUI:$TAG_LATEST" \
			"$IMAGE_ALL:$TAG_STABLE" "$IMAGE_ALL:$TAG_LATEST"
	fi

	set +e
}

deploy () {
	set -e

	docker load -i images.tar

	if [[ $TYPE == "snapshot" ]]; then
		docker push $CI_REGISTRY_IMAGE/cli:snapshot
		docker push $CI_REGISTRY_IMAGE/gui:snapshot
		docker push $CI_REGISTRY_IMAGE:snapshot
	else
		docker push $CI_REGISTRY_IMAGE/cli:latest
		docker push $CI_REGISTRY_IMAGE/gui:latest
		docker push $CI_REGISTRY_IMAGE:latest
		docker push $CI_REGISTRY_IMAGE/cli:$CI_COMMIT_REF_NAME
		docker push $CI_REGISTRY_IMAGE/gui:$CI_COMMIT_REF_NAME
		docker push $CI_REGISTRY_IMAGE:$CI_COMMIT_REF_NAME
	fi

	set +e
}
